

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (inaki@blueleaf.cc)

AtkHyperlinkImpl allows AtkObjects to refer to their associated
AtkHyperlink instance, if one exists.  AtkHyperlinkImpl differs
from AtkHyperlink in that AtkHyperlinkImpl is an interface, whereas
AtkHyperlink is a object type.  The AtkHyperlinkImpl interface
allows a client to query an AtkObject for the availability of an
associated AtkHyperlink instance, and obtain that instance.  It is
thus particularly useful in cases where embedded content or inline
content within a text object is present, since the embedding text
object implements AtkHypertext and the inline\/embedded objects are
exposed as children which implement AtkHyperlinkImpl, in addition
to their being obtainable via AtkHypertext:getLink followed by
AtkHyperlink:getObject.

The AtkHyperlinkImpl interface should be supported by objects
exposed within the hierarchy as children of an AtkHypertext
container which correspond to \"links\" or embedded content within
the text.  HTML anchors are not, for instance, normally exposed
this way, but embedded images and components which appear inline in
the content of a text object are. The AtkHyperlinkIface interface
allows a means of determining which children are hyperlinks in this
sense of the word, and for obtaining their corresponding
AtkHyperlink object, from which the embedding range, URI, etc. can
be obtained.

To some extent this interface exists because, for historical
reasons, AtkHyperlink was defined as an object type, not an
interface.  Thus, in order to interact with AtkObjects via
AtkHyperlink semantics, a new interface was required.
-}

#define ENABLE_OVERLOADING (MIN_VERSION_haskell_gi_overloading(1,0,0) \
       && !defined(__HADDOCK_VERSION__))

module GI.Atk.Interfaces.HyperlinkImpl
    ( 

-- * Exported types
    HyperlinkImpl(..)                       ,
    noHyperlinkImpl                         ,
    IsHyperlinkImpl                         ,


 -- * Methods
-- ** getHyperlink #method:getHyperlink#

#if ENABLE_OVERLOADING
    HyperlinkImplGetHyperlinkMethodInfo     ,
#endif
    hyperlinkImplGetHyperlink               ,




    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL

import {-# SOURCE #-} qualified GI.Atk.Objects.Hyperlink as Atk.Hyperlink

-- interface HyperlinkImpl 
-- | Memory-managed wrapper type.
newtype HyperlinkImpl = HyperlinkImpl (ManagedPtr HyperlinkImpl)
-- | A convenience alias for `Nothing` :: `Maybe` `HyperlinkImpl`.
noHyperlinkImpl :: Maybe HyperlinkImpl
noHyperlinkImpl = Nothing

#if ENABLE_OVERLOADING
type instance O.SignalList HyperlinkImpl = HyperlinkImplSignalList
type HyperlinkImplSignalList = ('[ ] :: [(Symbol, *)])

#endif

-- | Type class for types which implement `HyperlinkImpl`.
class ManagedPtrNewtype a => IsHyperlinkImpl a
instance IsHyperlinkImpl HyperlinkImpl
-- XXX Wrapping a foreign struct/union with no known destructor or size, leak?
instance WrappedPtr HyperlinkImpl where
    wrappedPtrCalloc = return nullPtr
    wrappedPtrCopy = return
    wrappedPtrFree = Nothing


#if ENABLE_OVERLOADING
type family ResolveHyperlinkImplMethod (t :: Symbol) (o :: *) :: * where
    ResolveHyperlinkImplMethod "getHyperlink" o = HyperlinkImplGetHyperlinkMethodInfo
    ResolveHyperlinkImplMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveHyperlinkImplMethod t HyperlinkImpl, O.MethodInfo info HyperlinkImpl p) => OL.IsLabel t (HyperlinkImpl -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#else
    fromLabel _ = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#endif

#endif

-- method HyperlinkImpl::get_hyperlink
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "impl", argType = TInterface (Name {namespace = "Atk", name = "HyperlinkImpl"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GObject instance that implements AtkHyperlinkImplIface", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Just (TInterface (Name {namespace = "Atk", name = "Hyperlink"}))
-- throws : False
-- Skip return : False

foreign import ccall "atk_hyperlink_impl_get_hyperlink" atk_hyperlink_impl_get_hyperlink :: 
    Ptr HyperlinkImpl ->                    -- impl : TInterface (Name {namespace = "Atk", name = "HyperlinkImpl"})
    IO (Ptr Atk.Hyperlink.Hyperlink)

{- |
Gets the hyperlink associated with this object.

/Since: 1.12/
-}
hyperlinkImplGetHyperlink ::
    (B.CallStack.HasCallStack, MonadIO m, IsHyperlinkImpl a) =>
    a
    {- ^ /@impl@/: a 'GI.GObject.Objects.Object.Object' instance that implements AtkHyperlinkImplIface -}
    -> m Atk.Hyperlink.Hyperlink
    {- ^ __Returns:__ an AtkHyperlink object which points to this
implementing AtkObject. -}
hyperlinkImplGetHyperlink impl = liftIO $ do
    impl' <- unsafeManagedPtrCastPtr impl
    result <- atk_hyperlink_impl_get_hyperlink impl'
    checkUnexpectedReturnNULL "hyperlinkImplGetHyperlink" result
    result' <- (wrapObject Atk.Hyperlink.Hyperlink) result
    touchManagedPtr impl
    return result'

#if ENABLE_OVERLOADING
data HyperlinkImplGetHyperlinkMethodInfo
instance (signature ~ (m Atk.Hyperlink.Hyperlink), MonadIO m, IsHyperlinkImpl a) => O.MethodInfo HyperlinkImplGetHyperlinkMethodInfo a signature where
    overloadedMethod _ = hyperlinkImplGetHyperlink

#endif


