

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (inaki@blueleaf.cc)

The AtkImplementor interface is implemented by objects for which
AtkObject peers may be obtained via calls to
iface->(ref_accessible)(implementor);
-}

#define ENABLE_OVERLOADING (MIN_VERSION_haskell_gi_overloading(1,0,0) \
       && !defined(__HADDOCK_VERSION__))

module GI.Atk.Interfaces.ImplementorIface
    ( 

-- * Exported types
    ImplementorIface(..)                    ,
    noImplementorIface                      ,
    IsImplementorIface                      ,


    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL


-- interface ImplementorIface 
-- | Memory-managed wrapper type.
newtype ImplementorIface = ImplementorIface (ManagedPtr ImplementorIface)
-- | A convenience alias for `Nothing` :: `Maybe` `ImplementorIface`.
noImplementorIface :: Maybe ImplementorIface
noImplementorIface = Nothing

#if ENABLE_OVERLOADING
type instance O.SignalList ImplementorIface = ImplementorIfaceSignalList
type ImplementorIfaceSignalList = ('[ ] :: [(Symbol, *)])

#endif

-- | Type class for types which implement `ImplementorIface`.
class ManagedPtrNewtype a => IsImplementorIface a
instance IsImplementorIface ImplementorIface
-- XXX Wrapping a foreign struct/union with no known destructor or size, leak?
instance WrappedPtr ImplementorIface where
    wrappedPtrCalloc = return nullPtr
    wrappedPtrCopy = return
    wrappedPtrFree = Nothing


#if ENABLE_OVERLOADING
type family ResolveImplementorIfaceMethod (t :: Symbol) (o :: *) :: * where
    ResolveImplementorIfaceMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveImplementorIfaceMethod t ImplementorIface, O.MethodInfo info ImplementorIface p) => OL.IsLabel t (ImplementorIface -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#else
    fromLabel _ = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#endif

#endif


