

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (inaki@blueleaf.cc)

A set of ATK utility functions which are used to support event
registration of various types, and obtaining the \'root\' accessible
of a process and information about the current ATK implementation
and toolkit version.
-}

#define ENABLE_OVERLOADING (MIN_VERSION_haskell_gi_overloading(1,0,0) \
       && !defined(__HADDOCK_VERSION__))

module GI.Atk.Objects.Util
    ( 

-- * Exported types
    Util(..)                                ,
    IsUtil                                  ,
    toUtil                                  ,
    noUtil                                  ,


    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL

import qualified GI.GObject.Objects.Object as GObject.Object

-- | Memory-managed wrapper type.
newtype Util = Util (ManagedPtr Util)
foreign import ccall "atk_util_get_type"
    c_atk_util_get_type :: IO GType

instance GObject Util where
    gobjectType = c_atk_util_get_type
    

-- | Type class for types which can be safely cast to `Util`, for instance with `toUtil`.
class (GObject o, O.IsDescendantOf Util o) => IsUtil o
instance (GObject o, O.IsDescendantOf Util o) => IsUtil o

instance O.HasParentTypes Util
type instance O.ParentTypes Util = '[GObject.Object.Object]

-- | Cast to `Util`, for types for which this is known to be safe. For general casts, use `Data.GI.Base.ManagedPtr.castTo`.
toUtil :: (MonadIO m, IsUtil o) => o -> m Util
toUtil = liftIO . unsafeCastTo Util

-- | A convenience alias for `Nothing` :: `Maybe` `Util`.
noUtil :: Maybe Util
noUtil = Nothing

#if ENABLE_OVERLOADING
type family ResolveUtilMethod (t :: Symbol) (o :: *) :: * where
    ResolveUtilMethod "bindProperty" o = GObject.Object.ObjectBindPropertyMethodInfo
    ResolveUtilMethod "bindPropertyFull" o = GObject.Object.ObjectBindPropertyFullMethodInfo
    ResolveUtilMethod "forceFloating" o = GObject.Object.ObjectForceFloatingMethodInfo
    ResolveUtilMethod "freezeNotify" o = GObject.Object.ObjectFreezeNotifyMethodInfo
    ResolveUtilMethod "getv" o = GObject.Object.ObjectGetvMethodInfo
    ResolveUtilMethod "isFloating" o = GObject.Object.ObjectIsFloatingMethodInfo
    ResolveUtilMethod "notify" o = GObject.Object.ObjectNotifyMethodInfo
    ResolveUtilMethod "notifyByPspec" o = GObject.Object.ObjectNotifyByPspecMethodInfo
    ResolveUtilMethod "ref" o = GObject.Object.ObjectRefMethodInfo
    ResolveUtilMethod "refSink" o = GObject.Object.ObjectRefSinkMethodInfo
    ResolveUtilMethod "runDispose" o = GObject.Object.ObjectRunDisposeMethodInfo
    ResolveUtilMethod "stealData" o = GObject.Object.ObjectStealDataMethodInfo
    ResolveUtilMethod "stealQdata" o = GObject.Object.ObjectStealQdataMethodInfo
    ResolveUtilMethod "thawNotify" o = GObject.Object.ObjectThawNotifyMethodInfo
    ResolveUtilMethod "unref" o = GObject.Object.ObjectUnrefMethodInfo
    ResolveUtilMethod "watchClosure" o = GObject.Object.ObjectWatchClosureMethodInfo
    ResolveUtilMethod "getData" o = GObject.Object.ObjectGetDataMethodInfo
    ResolveUtilMethod "getProperty" o = GObject.Object.ObjectGetPropertyMethodInfo
    ResolveUtilMethod "getQdata" o = GObject.Object.ObjectGetQdataMethodInfo
    ResolveUtilMethod "setData" o = GObject.Object.ObjectSetDataMethodInfo
    ResolveUtilMethod "setProperty" o = GObject.Object.ObjectSetPropertyMethodInfo
    ResolveUtilMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveUtilMethod t Util, O.MethodInfo info Util p) => OL.IsLabel t (Util -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#else
    fromLabel _ = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#endif

#endif

#if ENABLE_OVERLOADING
instance O.HasAttributeList Util
type instance O.AttributeList Util = UtilAttributeList
type UtilAttributeList = ('[ ] :: [(Symbol, *)])
#endif

#if ENABLE_OVERLOADING
#endif

#if ENABLE_OVERLOADING
type instance O.SignalList Util = UtilSignalList
type UtilSignalList = ('[ '("notify", GObject.Object.ObjectNotifySignalInfo)] :: [(Symbol, *)])

#endif


